@@ -110,26 +110,6 @@ def jsonattribute(name, **kw):
     return __outer_wr
 
 
-class Wrapper(object):
-    def __init__(self, obj):
-        self._wrapped_object = obj
-        self._jsonrpc_instance = True
-
-    def __call__(self, *args, **argv):
-        self._wrapped_object(*args, **argv)
-
-    def __getattr__(self, attr):
-        if attr in self.__dict__:
-            return getattr(self, attr)
-        else:
-            return getattr(self._wrapped_object, attr)
-
-def jsoninst(inst):
-    wrapped = Wrapper(inst)
-    wrapped._jsonrpc_method = True
-    return wrapped
-
-
 def isrpcmethod(obj):
     return getattr(obj, '_jsonrpc_method', False)
 
@@ -147,8 +127,14 @@ class RPCBase(object):
     log = logging.getLogger(__name__)
     def __init__(self):
         self._jsonrpc_method = True
+        self._jsonrpc_instance = True
         self.methods = self._get_methods()
 
+    def __setattr__(self, attr, value):
+        object.__setattr__(self, attr, value)
+        if isinstance(value, RPCBase):
+            self.methods = self._get_methods()
+
     def add_method(self, name, meth):
         self.log.debug("adding %s as %s", meth, name)
 
@@ -158,15 +144,16 @@ class RPCBase(object):
 
         if hasattr(self, name):
             raise AttributeError("Method `%s` already defined!" % name)
-        setattr(self, name, jsoninst(meth))
+        setattr(self, name, meth)
 
-        self.methods = self._get_methods()
 
     def _get_methods(self, obj = None):
         if obj is None:
             obj = self
         meths = {}
         for attrName in dir(obj):
+            if attrName.startswith("_"):
+                continue
             attr = getattr(obj, attrName)
             if not isrpcmethod(attr):
                 continue
